@@ -2,7 +2,7 @@ import copy
 
 from arangodb.api import Collection
 from arangodb.orm.fields import ModelField
-from arangodb.query.advanced import Query
+from arangodb.query.advanced import Query, Traveser
 from arangodb.query.simple import SimpleQuery
 
 
@@ -17,10 +17,30 @@ class CollectionQueryset(object):
         self._manager = manager
         self._collection = manager._model_class.collection_instance
         self._query = Query()
+        # Relations
+        self._is_working_on_relations = False
+        self._relations_start_model = None
+        self._relations_end_model = None
+        self._relations_relation_collection = None
+        self._related_model_class = None
         # Cache
         self._has_cache = False
         self._cache = []
 
+    def get_field_relations(self, relation_collection, related_model_class, start_model=None, end_model=None):
+        """
+        """
+
+        self._is_working_on_relations = True
+        self._has_cache = False
+
+        self._relations_start_model = start_model
+        self._relations_end_model = end_model
+        self._relations_relation_collection = relation_collection
+        self._related_model_class = related_model_class
+
+        return self
+
     def all(self):
         """
         """
@@ -61,10 +81,34 @@ class CollectionQueryset(object):
         self._cache = [] # TODO: Check how to best clear this list
         self._has_cache = True
 
-        result = self._query.execute()
+        if self._is_working_on_relations:
+
+            start_model = self._relations_start_model
+            end_model = self._relations_end_model
+            relation_collection = self._relations_relation_collection
+
+            if start_model:
+                found_relations = Traveser.follow(
+                    start_vertex=start_model.document.id,
+                    edge_collection=relation_collection,
+                    direction='outbound'
+                )
+            else:
+                found_relations = Traveser.follow(
+                    start_vertex=end_model.document.id,
+                    edge_collection=relation_collection,
+                    direction='inbound'
+                )
+
+            result = found_relations
+            result_class = self._related_model_class
+
+        else:
+            result = self._query.execute()
+            result_class = self._manager._model_class
 
         for doc in result:
-            model = self._manager._create_model_from_doc(doc=doc)
+            model = self._manager._create_model_from_doc(doc=doc, model_class=result_class)
             self._cache.append(model)
 
     def __getitem__(self, item):
@@ -118,7 +162,7 @@ class CollectionModelManager(object):
 
         return queryset
 
-    def _create_model_from_doc(self, doc):
+    def _create_model_from_doc(self, doc, model_class=None):
         """
         """
 
@@ -126,16 +170,19 @@ class CollectionModelManager(object):
 
         attributes = doc.get_attributes()
 
-        model = self._create_model_from_dict(attribute_dict=attributes)
+        model = self._create_model_from_dict(attribute_dict=attributes, model_class=model_class)
         model.document = doc
 
         return model
 
-    def _create_model_from_dict(self, attribute_dict):
+    def _create_model_from_dict(self, attribute_dict, model_class=None):
         """
         """
 
-        model = self._model_class()
+        if model_class:
+            model = model_class()
+        else:
+            model = self._model_class()
 
         attributes = attribute_dict
         for attribute_name in attributes:
@@ -143,7 +190,7 @@ class CollectionModelManager(object):
             if not attribute_name.startswith('_'):
                 field = model.get_field(name=attribute_name)
                 attribute_value = attributes[attribute_name]
-                field._model_instance = self._model_class
+                field._model_instance = model
                 field.loads(attribute_value)
 
         return model
@@ -156,6 +203,7 @@ class CollectionModel(object):
 
     objects = CollectionModelManager
 
+    _model_meta_data = None
     _instance_meta_data = None
 
     class RequiredFieldNoValue(Exception):
@@ -169,6 +217,29 @@ class CollectionModel(object):
         def __init__(self):
             self._fields = {}
 
+    @classmethod
+    def get_collection_fields(cls):
+        """
+        """
+
+        fields = []
+
+        for attribute in dir(cls):
+
+            attr_val = getattr(cls, attribute)
+            attr_cls = attr_val.__class__
+
+            # If it is a model field, call on init
+            if issubclass(attr_cls, ModelField):
+                fields.append(attr_val)
+
+        model_fields = cls._model_meta_data._fields
+        for field_key in  model_fields:
+            field = model_fields[field_key]
+            fields.append(field)
+
+        return fields
+
     @classmethod
     def init(cls):
         """
@@ -176,11 +247,13 @@ class CollectionModel(object):
 
         name = cls.get_collection_name()
 
+        # Set type
         try:
             collection_type = getattr(cls, 'collection_type')
         except:
             collection_type = 2
 
+        # Create collection
         try:
             cls.collection_instance = Collection.create(name=name, type=collection_type)
         except:
@@ -192,11 +265,22 @@ class CollectionModel(object):
         except:
             pass # This is the case if init was called more than once
 
+        # Create meta data for collection
+        cls._model_meta_data = cls.MetaDataObj()
+
+        # Go through all fields
+        for attribute in cls.get_collection_fields():
+            attribute.on_init(cls)
+
     @classmethod
     def destroy(cls):
         """
         """
 
+        # Go through all fields
+        for attribute in cls.get_collection_fields():
+            attribute.on_destroy(cls)
+
         name = cls.get_collection_name()
         Collection.remove(name=name)
 
@@ -220,20 +304,20 @@ class CollectionModel(object):
         self._instance_meta_data = CollectionModel.MetaDataObj()
         self.document = self.collection_instance.create_document()
 
-        for attribute in self._get_fields():
+        fields = self._get_fields()
+        for attribute in fields:
 
-            attr_val = getattr(self, attribute)
-            attr_cls = attr_val.__class__
+            attr_val = fields[attribute]
 
             # Only attributes which are fields are being copied
-            if issubclass(attr_cls, ModelField):
-                # Copy field with default config
-                field = copy.deepcopy(attr_val)
-                # Trigger on create so the field knows it
-                field.on_create()
-                # Save the new field in the meta data
-                self._instance_meta_data._fields[attribute] = field
-
+            # Copy field with default config
+            field = copy.deepcopy(attr_val)
+            # Set model instance on the field
+            field._model_instance = self
+            # Trigger on create so the field knows it
+            field.on_create(model_instance=self)
+            # Save the new field in the meta data
+            self._instance_meta_data._fields[attribute] = field
 
     def save(self):
         """
@@ -241,29 +325,41 @@ class CollectionModel(object):
 
         all_fields = self._instance_meta_data._fields
 
+        later_saved_fields = []
+
         for field_name in all_fields:
 
             local_field = all_fields[field_name]
-            is_field_required = local_field.required
 
-            if field_name in self._instance_meta_data._fields:
-                # Get field
-                field = self._instance_meta_data._fields[field_name]
-                # Validate content by field
-                field.validate()
-                # Get content
-                field_value = field.dumps()
-            else:
-                if not is_field_required:
-                    field_value = None
+            # Check if the field is saved in this collection
+            if local_field.__class__.is_saved_in_model:
+
+                is_field_required = local_field.required
+
+                if field_name in self._instance_meta_data._fields:
+                    # Get field
+                    field = self._instance_meta_data._fields[field_name]
+                    # Validate content by field
+                    field.validate()
+                    # Get content
+                    field_value = field.dumps()
                 else:
-                    raise CollectionModel.RequiredFieldNoValue()
+                    if not is_field_required:
+                        field_value = None
+                    else:
+                        raise CollectionModel.RequiredFieldNoValue()
 
+                self.document.set(key=field_name, value=field_value)
 
-            self.document.set(key=field_name, value=field_value)
+            # This field is saved somewhere else
+            else:
+                later_saved_fields.append(local_field)
 
         self.document.save()
 
+        for later_field in later_saved_fields:
+            later_field.on_save(self)
+
     def get_field(self, name):
         """
         """
@@ -284,6 +380,11 @@ class CollectionModel(object):
             if issubclass(attr_cls, ModelField):
                 fields[attribute] = attr_val
 
+        model_fields = self.__class__._model_meta_data._fields
+        for field_key in  model_fields:
+            field = model_fields[field_key]
+            fields[field_key] = field
+
         return fields
 
     def __getattribute__(self, item):
